/******************************************************************************
* Include files
******************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/atomic.h>
#include "usart.h"

/******************************************************************************
* Macros
******************************************************************************/
#define F_CPU  8000000UL  // 8 MHz

#include <avr/delay.h>

#define BAUD 19200
#define MYUBRR F_CPU/16/BAUD-1
/******************************************************************************
* Constants
******************************************************************************/


/******************************************************************************
* Global Variables
******************************************************************************/
static volatile usartRxBuff_t USART0_RxBuff;

/******************************************************************************
* External Variables
******************************************************************************/


/******************************************************************************
* Local Function Declarations
******************************************************************************/


/******************************************************************************
* Local Function Definitions
******************************************************************************/


/******************************************************************************
* Global Function Definitions
******************************************************************************/
/******************************************************************************
* Function:      void USART0_Init (uint16_t baud)
* Description:   Initialize USART0 with 8 data, no parity, 1 stop bit
* Input:         Calcuated baud rate register
* Output:        
* Notes:
******************************************************************************/
void USART0_Init (uint16_t baud)
{
	/* Set baud rate */
	UBRR0H = (uint16_t) (baud>>8);
	UBRR0L = (uint16_t) baud;
	/* RX Complete Interrupt Enable */
	UCSR0B = (1<<RXCIE0);
	/* Set frame format: 8data, no parity & 1 stop bits */
	UCSR0C = (0<<UMSEL0) | (0<<UPM0) | (0<<UPM1) | (0<<USBS0) | (3<<UCSZ0);
	/* Enable receiver and transmitter */
	UCSR0B |= (1<<RXEN0) | (1<<TXEN0);
}

/******************************************************************************
* Function:      void USART0_TransmitByte (uint8_t data)
* Description:   Send one byte to USART0
* Input:         Transmitted byte
* Output:       
* Notes:
******************************************************************************/
void USART0_TransmitByte (uint8_t data)
{
	/* Wait for empty transmit buffer */
	while ( ! ( UCSR0A & (1<<UDRE0)));
	/* Put data into buffer, sends the data */
	UDR0 = data;
}

/******************************************************************************
* Function:      void USART0_TransmitString (char *data)
* Description:   Send string to USART0
* Input:         Transmitted bytes
* Output:        Transmitted byte count
* Notes:
******************************************************************************/
uint16_t USART0_TransmitString (char *data)
{
	uint16_t i=0;
	
	while(data[i]!='\0')
		USART0_TransmitByte(data[i++]);
		
	return i;
}

/******************************************************************************
* Function:      void USART0_TransmitData (uint8_t *data, uint8_t length)
* Description:   Send bytes to USART0
* Input:         Transmitted bytes, length of data
* Output:        
* Notes:
******************************************************************************/
void USART0_TransmitData (uint8_t *data, uint8_t length)
{
	uint8_t i;
	
	for (i=0;i<length;i++)
		USART0_TransmitByte(data[i]);
}

/******************************************************************************
* Function:      void USART0_Flush (void)
* Description:   Flushes the receive FIFO buffer
* Input:
* Output:        
* Notes:        Use it, if the buffer has to be flushed during normal
*               operation, due to for instance an error condition
******************************************************************************/
void USART0_Flush (void)
{
	uint8_t dummy;
	while (UCSR0A & (1<<RXC0) ) dummy = UDR0;
}

uint8_t USART0_RxCount(void)
{
	return USART0_RxBuff.CharCount;
}

uint8_t USART0_RecieveByte(void)
{
	uint8_t rb;
	
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
	  if (USART0_RxBuff.CharCount)
	  {
	    rb = USART0_RxBuff.Buff[USART0_RxBuff.ReadIndex++];
		USART0_RxBuff.ReadIndex &= USART_RXBUFF_SIZE - 1;
		USART0_RxBuff.CharCount--;
		return rb;
	  }
	}
	
	return 0;
}
/******************************************************************************
* Interrupt Routines
******************************************************************************/
ISR (USART0_RX_vect)
{
	uint8_t rx_data;
	
	rx_data = UDR0;
	USART0_RxBuff.Buff[USART0_RxBuff.WriteIndex++] = rx_data;
	USART0_RxBuff.WriteIndex &= USART_RXBUFF_SIZE - 1;
	USART0_RxBuff.CharCount++;
}




